#!/usr/bin/env python

# Author                 Version      Date
# -----------------------------------------------
# Martin Still (SAAO)    1.0          02 Aug 2007

# saltstring 

import saltprint

# perform string functions on a list
# -----------------------------------------------------------

def listfunc(oldlist,proc,logfile):

    status = 0
    newlist = []
    if (proc == 'upper'):
	for value in oldlist:
	    newlist.append(value.upper())
    elif (proc == 'lower'):
	for value in oldlist:
	    newlist.append(value.lower())
    elif (proc == 'lstrip'):
	for value in oldlist:
	    newlist.append(value.lstrip())
    elif (proc == 'rstrip'):
	for value in oldlist:
	    newlist.append(value.rstrip())
    elif (proc == 'clean'):
	for value in oldlist:
	    newlist.append(value.lstrip().rstrip().upper())
    else:
	message = 'ERROR: SALTSTRING.LISTFUNC: unknown string function ' + proc
	status = saltprint.err(logfile,message)

    return newlist, status

# extract file number from file name
# -----------------------------------------------------------

def filenumber(filename):

    status = 0
    try:
	fileno = int(filename.rstrip()[9:-5])
    except:
	message = 'ERROR: SALTSTRING.FILENUMBER -- could not extract file number from filename' + filename
	status = saltprint.err(logfile,message)

    return fileno, status

# construct file name from file number
# -----------------------------------------------------------

def filename(prefix,obsdate,filenum):

    status = 0
    try:
	if (filenum < 10):
	    name = '000' + str(filenum)
	elif (filenum > 9 and filenum < 100):
	    name = '00' + str(filenum)
	elif (filenum > 99 and filenum < 1000):
	    name = '0' + str(filenum)
	else:
	    name = str(filenum)
	name = prefix + obsdate + name + '.fits'
    except:
	message = 'ERROR: SALTSTRING.FILENAME -- could not construct file name from file number' + filenum
	status = saltprint.err(logfile,message)

    return name, status

# find common string in two separate strings
# -----------------------------------------------------------

def extract(a,b):

    m = min(len(a),len(b))
    for i in range(m):
	if a[i] != b[i]:
	    return a[:i]

    return a[:m]

# extract x and y ranges from SEC keyword value
# -----------------------------------------------------------

def secsplit(sec,file,logfile):

    status = 0
    x = [None] * 2
    y = [None] * 2
    try:
	sec = sec.strip('[').strip(']')
	ranges = sec.split(',')
	x[0] = int(ranges[0].split(':')[0])
	x[1] = int(ranges[0].split(':')[1])
	y[0] = int(ranges[1].split(':')[0])
	y[1] = int(ranges[1].split(':')[1])
    except:
	message = 'ERROR: SALTSTRING.SECSPLIT -- failed to split SEC keyword from ' + file
	status = saltprint.err(logfile,status)

    return x, y, status
