#!/usr/bin/env python

# Author                 Version      Date
# -----------------------------------------------
# Martin Still (SAAO)    1.0          21 Nov 2006

# salttime...

# -----------------------------------------------
# Convert UTC to Julian Date

def juliandate(year,month,day,hour,min,sec):

    tmp = float(day) + float(hour) / 24. + float(min) / 1440. + sec / 86400.
    if (month < 3): 
	year -= 1
	month += 12
    a = year / 100
    b = 2 - a + a / 4
    c = int(365.25 * float(year))
    d = int(30.6001 * float(month + 1))
    julian = float(b + c + d) + tmp + 1720994.5 - 2453371.5 # midnight 2005/01/01

    return julian

# -----------------------------------------------
# Convert UTC to seconds

def utc2sec(hour,min,sec):

    time = float(hour) * 3600 + float(min) * 60 + sec

    return time

# -----------------------------------------------
# Convert TIME-OBS to hours

def time_obs2hr(time_obs):

    try:
      hour = float(time_obs[0:2]) + float(time_obs[3:5]) / 60 + float(time_obs[6:12]) / 3600
    except ValueError:
      hour = 99.999

    return hour

# -----------------------------------------------
# Convert from sexigesimal to decimal formats

def sex2dec(sex_value):
    value_arr=sex_value.split(':')
    dec_value=float(value_arr[0])+float(value_arr[1])/60.00+float(value_arr[2])/3600.0
    return dec_value
	


# -----------------------------------------------
# Convert TIME-OBS to YYYYMMDD

def date_obs2yyyymmdd(date_obs):
    monthName = {'01':'Jan', '02':'Feb', '03':'Mar', '04':'Apr', '05':'May', '06':'Jun',
              '07':'Jul','08':'Aug','09':'Sep','10':'Oct','11':'Nov','12':'Dec'}

    try:
      yyyymmdd = date_obs[0:4] + date_obs[5:7] + date_obs[8:10]
      year = date_obs[0:4]
      month=monthName[date_obs[5:7]]
      day = date_obs[8:10]
    except:
      year = '????'
      month = '???'
      day = '??'
      yyyymmdd='????????'

    date = day + ' ' + month + ' ' + year

    return yyyymmdd, date

# -----------------------------------------------
# Convert SALT date-time format to the DATETIME for mysql

def datatimeobs2DateTime(date):
    monthName = {'Jan':'01', 'Feb':'02', 'Mar':'03', 'Apr':'04', 'May':'05', 'Jun':'06',
              'Jul':'07','Aug':'08','Sep':'09','Oct':'10','Nov':'11','Dec':'12'}
    date=date.split()
    try:
	day=date[2]
	if int(day) < 10: day='0'+day
        mon=monthName[date[1]]
        year=date[4]
	time=date[3]
        newdate=year+'-'+mon+'-'+day+' '+time
    except:
	newdate=''

    return newdate
